# Generated by Django 5.1 on 2024-08-13 11:35

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('first_name', models.CharField(max_length=30)),
                ('last_name', models.CharField(max_length=30)),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('mobile_code', models.CharField(blank=True, max_length=30, null=True)),
                ('mobile_no', models.CharField(blank=True, max_length=30, null=True)),
                ('age', models.IntegerField(blank=True, null=True)),
                ('gender', models.PositiveIntegerField(blank=True, choices=[(1, 'Male'), (2, 'FEMALE'), (3, 'Other')], null=True)),
                ('profile_pic', models.FileField(blank=True, null=True, upload_to='profile_pic/')),
                ('role_id', models.PositiveIntegerField(blank=True, choices=[(1, 'Admin'), (2, 'User')], default=1, null=True)),
                ('social_id', models.CharField(blank=True, max_length=255, null=True)),
                ('social_type', models.PositiveIntegerField(blank=True, choices=[(1, 'Google'), (2, 'Instagram'), (3, 'Facebook'), (4, 'Apple')], null=True)),
                ('is_active', models.BooleanField(default=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('is_superuser', models.BooleanField(default=False)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateTimeField(auto_now=True, null=True)),
            ],
            options={
                'db_table': 'tbl_user',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('device_type', models.PositiveIntegerField(blank=True, choices=[(1, 'Android'), (2, 'IOS')], null=True)),
                ('device_name', models.CharField(blank=True, max_length=255, null=True)),
                ('device_token', models.TextField(blank=True, null=True)),
                ('ip_address', models.CharField(blank=True, max_length=255, null=True)),
                ('device_model', models.CharField(blank=True, max_length=255, null=True)),
                ('imei', models.CharField(blank=True, max_length=255, null=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'tbl_device',
                'managed': True,
            },
        ),
    ]



from .constants import *
from django.db import models
from django.contrib.auth.models import AbstractBaseUser


class User(AbstractBaseUser):
    first_name = models.CharField(max_length=30)
    last_name = models.CharField(max_length=30)
    email = models.EmailField(unique=True)
    mobile_code = models.CharField(max_length=30,blank=True,null=True)
    mobile_no = models.CharField(max_length=30,blank=True,null=True)
    age = models.IntegerField(blank=True,null=True)
    gender = models.PositiveIntegerField(choices=GENDER, null=True, blank=True)
    profile_pic = models.FileField(upload_to='profile_pic/', blank=True, null=True)
    role_id = models.PositiveIntegerField(default=ADMIN,choices=USER_ROLE,null=True, blank=True)
    social_id = models.CharField(max_length=255, null=True, blank=True)
    social_type = models.PositiveIntegerField(choices=SOCIAL_TYPE, null=True, blank=True)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    is_superuser = models.BooleanField(default=False)
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True,null=True, blank=True)
    
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []
    
    class Meta:
        managed = True
        db_table = 'tbl_user'

class Device(models.Model):
    user = models.ForeignKey('User',null=True,blank=True,on_delete=models.CASCADE)
    device_type = models.PositiveIntegerField(choices=DEVICE_TYPE,null=True,blank=True)
    device_name = models.CharField(max_length=255,null=True,blank=True)
    device_token = models.TextField(null=True,blank=True)
    ip_address = models.CharField(max_length=255,null=True,blank=True)
    device_model = models.CharField(max_length=255,null=True,blank=True)
    imei = models.CharField(max_length=255,null=True,blank=True)

    class Meta:
        managed = True
        db_table = 'tbl_device'

    def __str__(self):
        return str(self.device_name)


from .models import *
from .constants import *
from .serializers import *
from rest_framework.views import APIView 
from rest_framework.response import Response
from rest_framework import status,permissions
from rest_framework.authtoken.models import Token
from django.contrib.auth import authenticate, login
from django.contrib.auth.hashers import make_password


"""
SignUp Api
"""
class SignupApiView(APIView):
    permission_classes = (permissions.AllowAny,)

    def post(self, request, *args, **kwargs):
        if not request.data.get("first_name"):
            return Response({"message":"Please enter first name","status":status.HTTP_400_BAD_REQUEST},status=status.HTTP_400_BAD_REQUEST)
        if not request.data.get("last_name"):
            return Response({"message":"Please enter last name","status":status.HTTP_400_BAD_REQUEST},status=status.HTTP_400_BAD_REQUEST)
        if not request.data.get("mobile_no"):
            return Response({"message":"Please enter mobile number","status":status.HTTP_400_BAD_REQUEST},status=status.HTTP_400_BAD_REQUEST)
        if not request.data.get("email"):
            return Response({"message":"Please enter email","status":status.HTTP_400_BAD_REQUEST},status=status.HTTP_400_BAD_REQUEST)
        if not request.data.get("password"):
            return Response({"message":"Please enter password","status":status.HTTP_400_BAD_REQUEST},status=status.HTTP_400_BAD_REQUEST)

        user = User.objects.create(
            first_name = request.data.get("first_name"),
            last_name = request.data.get("last_name"),
            email = request.data.get("email"),
            mobile_no = request.data.get("mobile_no"),
            password = make_password(request.data.get("password")),
            role_id = request.data.get("role_id"),
            gender = request.data.get("gender"),
        )
          
        try:
            device = Device.objects.get(user = user)
        except Device.DoesNotExist:
            device = Device.objects.create(user = user)
        if request.data.get('device_type'):
            device.device_type = request.data.get('device_type')
        if request.data.get('device_name'):
            device.device_name = request.data.get('device_name')
        if request.data.get('device_token'):
            device.device_token = request.data.get('device_token')
        
        device.save()
        data = UserSerializer(user,context = {"request":request}).data
        return Response({"message":"User Registered Successfully","data":data},status=status.HTTP_200_OK)


"""
Login Api
"""
class LoginApiView(APIView):
    permission_classes = (permissions.AllowAny,)

    def post(self, request, *args, **kwargs):
        if not request.data.get("email"):
            return Response({"message":"Please enter email","status":status.HTTP_400_BAD_REQUEST})
        if not request.data.get("password"):
            return Response({"message":"Please enter password"},status=status.HTTP_400_BAD_REQUEST)

        user = authenticate(username=request.data.get("email"), password=request.data.get("password"))

        if not user:
            return Response({"message":"Invalid Login Credentials.", "status":status.HTTP_400_BAD_REQUEST})
        if not user.is_active:
            return Response({"message":"Your account has been deactivated. Please contact admin to activate your account.","status":status.HTTP_400_BAD_REQUEST})

        login(request, user)
        try:
            token = Token.objects.get(user = user)
        except:
            token = Token.objects.create(user = user)

        try:
            device = Device.objects.get(user = user)
        except Device.DoesNotExist:
            device = Device.objects.create(user = user)

        if request.data.get('device_type'):
            device.device_type = request.data.get('device_type')
        if request.data.get('device_name'):
            device.device_name = request.data.get('device_name')
        if request.data.get('device_token'):
            device.device_token = request.data.get('device_token')
        
        device.save()

        data = UserSerializer(user,context = {"request":request}).data
        data.update({"token":token.key})   
        return Response({"message":"Logged In Successfully","data":data,"status":status.HTTP_200_OK}, status=status.HTTP_200_OK)



from .constants import *
from django.db.models import Q
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend



User = get_user_model()

class EmailLoginBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, role_id=None, **kwargs):
        try:
            user = User.objects.filter(Q(email=username)|Q(mobile_no=username)).last()
        except User.DoesNotExist:
            return None
        if user.check_password(password):
            return user


from .models import *
from rest_framework.serializers import ModelSerializer



class UserSerializer(ModelSerializer):

    class Meta:
        model = User
        fields = ['first_name','last_name','email','mobile_code','mobile_no',
                  'age','gender','profile_pic','role_id','social_id','social_type',
                  'is_active','is_staff','is_superuser','created_on','updated_on']



"""
User role type name
"""
USER_ROLE = ((1, "Admin"),(2, "User"))
ADMIN = 1
USER = 2

"""
User Status 
"""
USER_STATUS = ((1, "Active"),(2,"Inactive"),(3,"Deleted"),(4, "Suspended"),(5,"Terminated"))
ACTIVE = 1
INACTIVE = 2
DELETED = 3
SUSPENDED = 4
TERMINATED = 5

"""
GENDER
"""
GENDER = ((1,'Male'), (2, 'FEMALE'), (3,'Other'))
MALE = 1
FEMALE = 2
OTHER = 3

"""
Social Logins
"""
SOCIAL_TYPE = ((1,'Google'),(2,'Instagram'),(3,'Facebook'),(4,'Apple'))
GOOGLE = 1
INSTAGRAM = 2
FACEBOOK = 3
APPLE = 4

"""
Device
"""
DEVICE_TYPE  = ((1,"Android"),(2,"IOS"))
ANDROID = 1
IOS = 2


from .views import *
from django.urls import re_path
from django.contrib import admin


admin.autodiscover()

app_name = 'accounts'

urlpatterns = [
    ## Authentication
    re_path(r'^signup/$', SignupApiView.as_view(), name='signup_api'),
    re_path(r'^login/$', LoginApiView.as_view(), name='login_api'),

]


from setuptools import setup, find_packages

setup(
    name='ns-django-custom-auth',
    version='0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django>=3.2',
    ],
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ],
)
